use config::{Config, ConfigError, File};
use directories::ProjectDirs;
use serde::{Deserialize, Serialize};
use std::fs;
use std::path::PathBuf;

#[derive(Debug, Deserialize, Serialize)]
pub struct JiraConfig {
    pub url: String,
    pub email: String,
    pub api_token: String,
}

impl JiraConfig {
    pub fn load() -> Result<Self, ConfigError> {
        let config_path = JiraConfig::get_config_path()?;

        if !config_path.exists() {
            return Err(ConfigError::NotFound(
                "Config file not found. Run 'init' command first.".to_string(),
            ));
        }

        let settings = Config::builder()
            .add_source(File::with_name(config_path.to_str().unwrap()))
            .build()?;

        settings.try_deserialize()
    }

    pub fn save(&self) -> Result<(), Box<dyn std::error::Error>> {
        let config_path = JiraConfig::get_config_path()?;

        // Ensure config directory exists
        if let Some(parent) = config_path.parent() {
            fs::create_dir_all(parent)?;
        }

        let toml = toml::to_string(self)?;
        fs::write(&config_path, toml)?;

        println!("Configuration saved to: {}", config_path.display());
        Ok(())
    }

    pub fn get_config_path() -> Result<PathBuf, ConfigError> {
        let proj_dirs = ProjectDirs::from("com", "runebaas", "jirac").ok_or_else(|| {
            ConfigError::NotFound("Could not determine config directory".to_string())
        })?;

        Ok(proj_dirs.config_dir().join("config.toml"))
    }

    pub async fn init(
        url: String,
        email: String,
        token: String,
    ) -> Result<(), Box<dyn std::error::Error>> {
        let config = JiraConfig {
            url,
            email,
            api_token: token,
        };

        config.save()?;
        Ok(())
    }
}
